import * as fs from "fs";
import { FSTree, md5Hex, walkSync } from "../helpers";
export default class OutputPatcher {
    constructor(outputPath) {
        this.outputPath = outputPath;
        this.entries = [];
        this.contents = new Map();
        this.lastTree = undefined;
        this.isUnchanged = (entryA, entryB) => {
            if (entryA.isDirectory() && entryB.isDirectory()) {
                return true;
            }
            if (entryA.mode === entryB.mode && entryA.checksum === entryB.checksum) {
                return true;
            }
            return false;
        };
    }
    // relativePath should be without leading '/' and use forward slashes
    add(relativePath, content) {
        this.entries.push(new Entry(this.outputPath, relativePath, md5Hex(content)));
        this.contents.set(relativePath, content);
    }
    patch() {
        try {
            this.lastTree = this._patch();
        }
        catch (e) {
            // walkSync(output);
            this.lastTree = undefined;
            throw e;
        }
        finally {
            this.entries = [];
            this.contents = new Map();
        }
    }
    _patch() {
        const entries = this.entries;
        let lastTree = this.lastTree;
        const isUnchanged = this.isUnchanged;
        const outputPath = this.outputPath;
        const contents = this.contents;
        const nextTree = FSTree.fromEntries(entries, { sortAndExpand: true });
        if (!lastTree) {
            lastTree = FSTree.fromEntries(walkSync.entries(outputPath));
        }
        const patch = lastTree.calculatePatch(nextTree, isUnchanged);
        patch.forEach(change => {
            var _a;
            const op = change[0];
            const path = change[1];
            const entry = change[2];
            switch (op) {
                case "mkdir":
                    // the expanded dirs don't have a base
                    fs.mkdirSync(outputPath + "/" + path);
                    break;
                case "rmdir":
                    // the expanded dirs don't have a base
                    fs.rmdirSync(outputPath + "/" + path);
                    break;
                case "unlink":
                    fs.unlinkSync(entry.fullPath);
                    break;
                case "create":
                case "change":
                    fs.writeFileSync(entry.fullPath, (_a = contents.get(path)) !== null && _a !== void 0 ? _a : "");
                    break;
                default:
                    throw new Error(`unrecognized case ${op}`);
            }
        });
        return nextTree;
    }
}
/* tslint:disable:max-classes-per-file */
class Entry {
    constructor(basePath, relativePath, checksum) {
        this.basePath = basePath;
        this.relativePath = relativePath;
        this.checksum = checksum;
        this.mode = 0;
        this.size = 0;
        this.mtime = new Date();
        this.fullPath = basePath + "/" + relativePath;
        this.checksum = checksum;
    }
    isDirectory() {
        return false;
    }
}
//# sourceMappingURL=output-patcher.js.map